@@ -1,6 +1,6 @@
-/* vi: set ts=2 shiftwidth=2 expandtab:
- *
+/*
  * Copyright (C) 2001-2006  Simon Baldwin (simon_baldwin@yahoo.com)
+ * Copyright (C) 2011       Kamil Ignacak (acerion@wp.pl)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,10 +12,11 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
+#define _BSD_SOURCE
 
 #include "../config.h"
 
@@ -36,7 +37,7 @@
 #endif
 
 #include "cw.h"
-#include "cwlib.h"
+#include "libcw.h"
 
 #include "i18n.h"
 #include "cmdline.h"
@@ -66,11 +67,23 @@ static void write_to_cw_sender (const char *format, ...)
  * command parser to values other than strictly TRUE or FALSE; all non-zero
  * values are equivalent to TRUE.
  */
+#if 0
 static int do_echo = TRUE,          /* Echo characters */
            do_errors = TRUE,        /* Print error messages to stderr */
            do_commands = TRUE,      /* Execute embedded commands */
            do_combinations = TRUE,  /* Execute [...] combinations */
            do_comments = TRUE;      /* Allow {...} as comments */
+#endif
+
+static cw_config_t *config = NULL;
+static const char *all_options = "s:|system,d:|device,"
+	"w:|wpm,t:|tone,v:|volume,"
+	"g:|gap,k:|weighting,"
+	"f:|infile,"
+	"e|noecho,m|nomessages,c|nocommands,o|nocombinations,p|nocomments,"
+	"h|help,V|version";
+static const char *argv0 = NULL;
+
 
 
 /*---------------------------------------------------------------------*/
@@ -87,7 +100,7 @@ static int do_echo = TRUE,          /* Echo characters */
 static void
 write_to_echo_stream (const char *format, ...)
 {
-  if (do_echo)
+  if (config->do_echo)
     {
       va_list ap;
 
@@ -101,7 +114,7 @@ write_to_echo_stream (const char *format, ...)
 static void
 write_to_message_stream (const char *format, ...)
 {
-  if (do_errors)
+  if (config->do_errors)
     {
       va_list ap;
 
@@ -188,19 +201,19 @@ parse_stream_query (FILE *stream)
       value = cw_get_weighting ();
       break;
     case CW_CMDV_ECHO:
-      value = do_echo;
+      value = config->do_echo;
       break;
     case CW_CMDV_ERRORS:
-      value = do_errors;
+      value = config->do_errors;
       break;
     case CW_CMDV_COMMANDS:
-      value = do_commands;
+      value = config->do_commands;
       break;
     case CW_CMDV_COMBINATIONS:
-      value = do_combinations;
+      value = config->do_combinations;
       break;
     case CW_CMDV_COMMENTS:
-      value = do_comments;
+      value = config->do_comments;
       break;
     }
 
@@ -250,23 +263,23 @@ parse_stream_cwquery (FILE *stream)
       format = _("%d PERCENT ");
       break;
     case CW_CMDV_ECHO:
-      value = do_echo;
+      value = config->do_echo;
       format = _("ECHO %s ");
       break;
     case CW_CMDV_ERRORS:
-      value = do_errors;
+      value = config->do_errors;
       format = _("ERRORS %s ");
       break;
     case CW_CMDV_COMMANDS:
-      value = do_commands;
+      value = config->do_commands;
       format = _("COMMANDS %s ");
       break;
     case CW_CMDV_COMBINATIONS:
-      value = do_combinations;
+      value = config->do_combinations;
       format = _("COMBINATIONS %s ");
       break;
     case CW_CMDV_COMMENTS:
-      value = do_comments;
+      value = config->do_comments;
       format = _("COMMENTS %s ");
       break;
     }
@@ -334,19 +347,19 @@ parse_stream_parameter (int c, FILE *stream)
       value_handler = cw_set_weighting;
       break;
     case CW_CMDV_ECHO:
-      do_echo = value;
+      config->do_echo = value;
       break;
     case CW_CMDV_ERRORS:
-      do_errors = value;
+      config->do_errors = value;
       break;
     case CW_CMDV_COMMANDS:
-      do_commands = value;
+      config->do_commands = value;
       break;
     case CW_CMDV_COMBINATIONS:
-      do_combinations = value;
+      config->do_combinations = value;
       break;
     case CW_CMDV_COMMENTS:
-      do_comments = value;
+      config->do_comments = value;
       break;
     }
 
@@ -489,17 +502,17 @@ parse_stream (FILE *stream)
            * Start a comment or combination, handle a command escape, or send
            * the character if none of these checks apply.
            */
-          if (do_comments && c == CW_COMMENT_START)
+          if (config->do_comments && c == CW_COMMENT_START)
             {
               state = COMMENT;
               write_to_echo_stream ("%c", c);
             }
-          else if (do_combinations && c == CW_COMBINATION_START)
+          else if (config->do_combinations && c == CW_COMBINATION_START)
             {
               state = COMBINATION;
               write_to_echo_stream ("%c", c);
             }
-          else if (do_commands && c == CW_CMD_ESCAPE)
+          else if (config->do_commands && c == CW_CMD_ESCAPE)
             parse_stream_command (stream);
           else
             send_cw_character (c, FALSE);
@@ -511,7 +524,7 @@ parse_stream (FILE *stream)
            * handle a command escape, or send the character if none of these
            * checks apply.
            */
-          if (do_comments && c == CW_COMMENT_START)
+          if (config->do_comments && c == CW_COMMENT_START)
             {
               state = NESTED_COMMENT;
               write_to_echo_stream ("%c", c);
@@ -521,7 +534,7 @@ parse_stream (FILE *stream)
               state = NONE;
               write_to_echo_stream ("%c", c);
             }
-          else if (do_commands && c == CW_CMD_ESCAPE)
+          else if (config->do_commands && c == CW_CMD_ESCAPE)
             parse_stream_command (stream);
           else
             {
@@ -554,336 +567,7 @@ parse_stream (FILE *stream)
 }
 
 
-/*---------------------------------------------------------------------*/
-/*  Command line mechanics                                             */
-/*---------------------------------------------------------------------*/
 
-/*
- * print_usage()
- *
- * Print out a brief message directing the user to the help function.
- */
-static void
-print_usage (const char *argv0)
-{
-  const char *format;
-
-  format = has_longopts ()
-    ? _("Try '%s --help' for more information.\n")
-    : _("Try '%s -h' for more information.\n");
-
-  fprintf (stderr, format, argv0);
-  exit (EXIT_FAILURE);
-}
-
-/*
- * print_help()
- *
- * Print out a brief page of help information.
- */
-static void
-print_help (const char *argv0)
-{
-  const char *format;
-  int min_speed, max_speed, min_frequency, max_frequency, min_volume,
-      max_volume, min_gap, max_gap, min_weighting, max_weighting;
-
-  cw_reset_send_receive_parameters ();
-  cw_get_speed_limits (&min_speed, &max_speed);
-  cw_get_frequency_limits (&min_frequency, &max_frequency);
-  cw_get_volume_limits (&min_volume, &max_volume);
-  cw_get_gap_limits (&min_gap, &max_gap);
-  cw_get_weighting_limits (&min_weighting, &max_weighting);
-
-  format = has_longopts ()
-    ? _("Usage: %s [options...]\n\n"
-      "  -s, --sound=SOURCE     generate sound on SOURCE"
-      " [default 'soundcard']\n"
-      "                         one of 's[oundcard]', 'c[onsole]',"
-      " or 'b[oth]'\n"
-      "  -x, --sdevice=SDEVICE  use SDEVICE for soundcard [default %s]\n"
-      "  -y, --mdevice=MDEVICE  use MDEVICE for sound mixer [default %s]\n"
-      "  -d, --cdevice=CDEVICE  use CDEVICE for sound ioctl [default %s]\n"
-      "  -f, --file=FILE        read from FILE [default stdin]\n")
-    : _("Usage: %s [options...]\n\n"
-      "  -s SOURCE   generate sound on SOURCE [default 'console']\n"
-      "              one of 'c[onsole]', 's[oundcard]', or 'b[oth]'\n"
-      "  -x SDEVICE  use SDEVICE for soundcard [default %s]\n"
-      "  -y MDEVICE  use MDEVICE for sound mixer [default %s]\n"
-      "  -d CDEVICE  use CDEVICE for sound ioctl [default %s]\n"
-      "  -f FILE     read from FILE [default stdin]\n");
-
-  printf (format, argv0,
-          cw_get_soundcard_file (),
-          cw_get_soundmixer_file (),
-          cw_get_console_file ());
-
-  format = has_longopts ()
-    ? _("  -w, --wpm=WPM          set initial words per minute [default %d]\n"
-      "                         valid WPM values are between %d and %d\n"
-      "  -t, --hz,--tone=HZ     set initial tone to HZ [default %d]\n"
-      "                         valid HZ values are between %d and %d\n"
-      "  -v, --volume=PERCENT   set initial volume to PERCENT [default %d]\n"
-      "                         valid PERCENT values are between %d and %d\n")
-    : _("  -w WPM      set initial words per minute [default %d]\n"
-      "              valid WPM values are between %d and %d\n"
-      "  -t HZ       set initial tone to HZ [default %d]\n"
-      "              valid HZ values are between %d and %d\n"
-      "  -v PERCENT  set initial volume to PERCENT [default %d]\n"
-      "              valid PERCENT values are between %d and %d\n");
-
-  printf (format,
-          cw_get_send_speed (), min_speed, max_speed,
-          cw_get_frequency (), min_frequency, max_frequency,
-          cw_get_volume (), min_volume, max_volume);
-
-  format = has_longopts ()
-    ? _("  -g, --gap=GAP          set extra gap between letters [default %d]\n"
-      "                         valid GAP values are between %d and %d\n"
-      "  -k, --weighting=WEIGHT set weighting to WEIGHT [default %d]\n"
-      "                         valid WEIGHT values are between %d and %d\n"
-      "  -e, --noecho           don't echo sending to stdout [default echo]\n"
-      "  -m, --nomessages       don't write messages to stderr"
-      " [default messages]\n")
-    : _("  -g GAP      set extra gap between letters [default %d]\n"
-      "              valid GAP values are between %d and %d\n"
-      "  -k WEIGHT   set weighting to WEIGHT [default %d]\n"
-      "              valid WEIGHT values are between %d and %d\n"
-      "  -e          don't echo sending to stdout [default echo]\n"
-      "  -m          don't write messages to stderr [default messages]\n");
-
-  printf (format,
-          cw_get_gap (), min_gap, max_gap,
-          cw_get_weighting (), min_weighting, max_weighting);
-
-  format = has_longopts ()
-    ? _("  -c, --nocommands       don't execute embedded commands"
-      " [default commands]\n"
-      "  -o, --nocombinations   don't allow [...] combinations"
-      " [default combos]\n"
-      "  -p, --nocomments       don't allow {...} comments [default comments]\n"
-      "  -h, --help             print this message\n"
-      "  -V, --version          output version information and exit\n\n")
-    : _("  -c          don't execute embedded commands [default commands]\n"
-      "  -o          don't allow [...] combinations [default combinations]\n"
-      "  -p          don't allow {...} comments [default comments]\n"
-      "  -h          print this message\n"
-      "  -V          output version information and exit\n\n");
-
-  printf (format);
-  exit (EXIT_SUCCESS);
-}
-
-
-/*
- * parse_command_line()
- *
- * Parse the command line options for initial values for the various
- * global and flag definitions.
- */
-static void
-parse_command_line (int argc, char *const argv[])
-{
-  const char *argv0;
-  int is_console = FALSE, is_soundcard = TRUE;
-  const char *console_device = NULL,
-             *soundcard_device = NULL, *mixer_device = NULL;
-  int option;
-  char *argument;
-
-  argv0 = program_basename (argv[0]);
-  while (get_option (argc, argv,
-                     _("s:|sound,d:|cdevice,x:|sdevice,y:|mdevice,f:|file,"
-                     "t:|tone,t:|hz,v:|volume,w:|wpm,g:|gap,k:|weighting,"
-                     "e|noecho,m|nomessages,c|nocommands,o|nocombinations,"
-                     "p|nocomments,h|help,V|version"),
-                     &option, &argument))
-    {
-      int intarg;
-
-      switch (option)
-        {
-        case 's':
-          if (strcoll (argument, _("console")) == 0
-              || strcoll (argument, _("c")) == 0)
-            {
-              is_console = TRUE;
-              is_soundcard = FALSE;
-            }
-          else if (strcoll (argument, _("soundcard")) == 0
-                   || strcoll (argument, _("s")) == 0)
-            {
-              is_console = FALSE;
-              is_soundcard = TRUE;
-            }
-          else if (strcoll (argument, _("both")) == 0
-                   || strcoll (argument, _("b")) == 0)
-            {
-              is_console = TRUE;
-              is_soundcard = TRUE;
-            }
-          else
-            {
-              fprintf (stderr, _("%s: invalid sound source\n"), argv0);
-              exit (EXIT_FAILURE);
-            }
-          break;
-
-        case 'd':
-          console_device = argument;
-          break;
-
-        case 'x':
-          soundcard_device = argument;
-          break;
-
-        case 'y':
-          mixer_device = argument;
-          break;
-
-        case 'f':
-          if (!freopen (argument, "r", stdin))
-            {
-              fprintf (stderr, _("%s: error opening input file\n"), argv0);
-              perror (argument);
-              exit (EXIT_FAILURE);
-            }
-          break;
-
-        case 't':
-          if (sscanf (argument, "%d", &intarg) != 1
-              || !cw_set_frequency (intarg))
-            {
-              fprintf (stderr, _("%s: invalid tone value\n"), argv0);
-              exit (EXIT_FAILURE);
-            }
-          break;
-
-        case 'v':
-          if (sscanf (argument, "%d", &intarg) != 1
-              || !cw_set_volume (intarg))
-            {
-              fprintf (stderr, _("%s: invalid volume value\n"), argv0);
-              exit (EXIT_FAILURE);
-            }
-          break;
-
-        case 'w':
-          if (sscanf (argument, "%d", &intarg) != 1
-              || !cw_set_send_speed (intarg))
-            {
-              fprintf (stderr, _("%s: invalid wpm value\n"), argv0);
-              exit (EXIT_FAILURE);
-            }
-          break;
-
-        case 'g':
-          if (sscanf (argument, "%d", &intarg) != 1
-              || !cw_set_gap (intarg))
-            {
-              fprintf (stderr, _("%s: invalid gap value\n"), argv0);
-              exit (EXIT_FAILURE);
-            }
-          break;
-
-        case 'i':
-          if (sscanf (argument, "%d", &intarg) != 1
-              || !cw_set_weighting (intarg))
-            {
-              fprintf (stderr, _("%s: invalid weighting value\n"), argv0);
-              exit (EXIT_FAILURE);
-            }
-          break;
-
-        case 'e':
-          do_echo = FALSE;
-          break;
-
-        case 'm':
-          do_errors = FALSE;
-          break;
-
-        case 'c':
-          do_commands = FALSE;
-          break;
-
-        case 'o':
-          do_combinations = FALSE;
-          break;
-
-        case 'p':
-          do_comments = FALSE;
-          break;
-
-        case 'h':
-          print_help (argv0);
-
-        case 'V':
-          printf (_("%s version %s, %s\n"),
-                  argv0, PACKAGE_VERSION, _(CW_COPYRIGHT));
-          exit (EXIT_SUCCESS);
-
-        case '?':
-          print_usage (argv0);
-
-        default:
-          fprintf (stderr, _("%s: getopts returned '%c'\n"), argv0, option);
-          exit (EXIT_FAILURE);
-        }
-    }
-  if (get_optind () != argc)
-    print_usage (argv0);
-
-  /* Deal with odd argument combinations. */
-  if (!is_console && console_device)
-    {
-      fprintf (stderr, _("%s: no console sound: -d invalid\n"), argv0);
-      print_usage (argv0);
-    }
-  else if (!is_soundcard)
-    {
-      if (soundcard_device)
-        {
-          fprintf (stderr, _("%s: no soundcard sound: -x invalid\n"), argv0);
-          print_usage (argv0);
-        }
-      else if (mixer_device)
-        {
-          fprintf (stderr, _("%s: no soundcard sound: -y invalid\n"), argv0);
-          print_usage (argv0);
-        }
-    }
-
-  /* First set up soundcard sound if required. */
-  if (is_soundcard)
-    {
-      if (soundcard_device)
-        cw_set_soundcard_file (soundcard_device);
-      if (!cw_is_soundcard_possible ())
-        {
-          fprintf (stderr, _("%s: cannot set up soundcard sound\n"), argv0);
-          perror (cw_get_soundcard_file ());
-          exit (EXIT_FAILURE);
-        }
-      if (mixer_device)
-        cw_set_soundmixer_file (mixer_device);
-    }
-  cw_set_soundcard_sound (is_soundcard);
-
-  /* Now set up console sound, if required. */
-  if (is_console)
-    {
-      if (console_device)
-        cw_set_console_file (console_device);
-      if (!cw_is_console_possible ())
-        {
-          fprintf (stderr, _("%s: cannot set up console sound\n"), argv0);
-          perror (cw_get_console_file ());
-          exit (EXIT_FAILURE);
-        }
-    }
-  cw_set_console_sound (is_console);
-}
 
 
 /*
@@ -891,36 +575,70 @@ parse_command_line (int argc, char *const argv[])
  *
  * Parse command line args, then produce CW output until end of file.
  */
-int
-main (int argc, char *const argv[])
+int main (int argc, char *const argv[])
 {
-  static const int SIGNALS[] = { SIGHUP, SIGINT, SIGQUIT, SIGPIPE, SIGTERM, 0 };
+	argv0 = program_basename(argv[0]);
 
-  int combined_argc, index;
-  char **combined_argv;
+	/* Set locale and message catalogs. */
+	i18n_initialize();
 
-  /* Set locale and message catalogs. */
-  i18n_initialize ();
+	/* Parse combined environment and command line arguments. */
+	int combined_argc;
+	char **combined_argv;
+	combine_arguments("CW_OPTIONS", argc, argv, &combined_argc, &combined_argv);
 
-  /* Parse combined environment and command line arguments. */
-  combine_arguments (_("CW_OPTIONS"),
-                     argc, argv, &combined_argc, &combined_argv);
-  parse_command_line (combined_argc, combined_argv);
+	config = cw_config_new();
+	if (!config) {
+		return EXIT_FAILURE;
+	}
+	config->is_cw = 1;
 
-  /* Set up signal handlers to exit on a range of signals. */
-  for (index = 0; SIGNALS[index] != 0; index++)
-    {
-      if (!cw_register_signal_handler (SIGNALS[index], SIG_DFL))
-        {
-          perror ("cw_register_signal_handler");
-          abort ();
-        }
-    }
+	if (!cw_process_argv(argc, argv, all_options, config)) {
+		fprintf(stderr, _("%s: failed to parse command line args\n"), argv0);
+		return EXIT_FAILURE;
+	}
+	if (!cw_config_is_valid(config)) {
+		fprintf(stderr, _("%s: inconsistent arguments\n"), argv0);
+		return EXIT_FAILURE;
+	}
 
-  /* Send stdin stream to CW parsing. */
-  parse_stream (stdin);
+	if (config->input_file) {
+		if (!freopen(config->input_file, "r", stdin)) {
+			fprintf(stderr, _("%s: %s\n"), argv0, strerror(errno));
+			fprintf(stderr, _("%s: error opening input file %s\n"), argv0, config->input_file);
+			return EXIT_FAILURE;
+		}
+	}
 
-  /* Await final tone completion before exiting. */
-  cw_wait_for_tone_queue ();
-  return EXIT_SUCCESS;
+	if (!cw_generator_new_from_config(config, argv0)) {
+		//fprintf(stderr, "%s: failed to create generator with device '%s'\n", argv0, config->audio_device);
+		return EXIT_FAILURE;
+	}
+
+	/* Set up signal handlers to exit on a range of signals. */
+	int index;
+	static const int SIGNALS[] = { SIGHUP, SIGINT, SIGQUIT, SIGPIPE, SIGTERM, 0 };
+	for (index = 0; SIGNALS[index] != 0; index++) {
+		if (!cw_register_signal_handler(SIGNALS[index], SIG_DFL)) {
+			fprintf(stderr, _("%s: can't register signal: %s\n"), argv0, strerror(errno));
+			return EXIT_FAILURE;
+		}
+	}
+
+	/* Start producing sine wave (amplitude of the wave will be
+	   zero as long as there are no characters to process). */
+	cw_generator_start();
+
+	/* Send stdin stream to CW parsing. */
+	parse_stream(stdin);
+
+	/* Await final tone completion before exiting. */
+	cw_wait_for_tone_queue();
+
+	cw_generator_stop();
+	cw_generator_delete();
+
+	cw_config_delete(&config);
+
+	return EXIT_SUCCESS;
 }
