@@ -1,6 +1,6 @@
-/* vi: set ts=2 shiftwidth=2 expandtab:
- *
+/*
  * Copyright (C) 2001-2006  Simon Baldwin (simon_baldwin@yahoo.com)
+ * Copyright (C) 2011       Kamil Ignacak (acerion@wp.pl)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,9 +12,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "../config.h"
@@ -39,7 +39,7 @@
 # include <strings.h>
 #endif
 
-#include "cwlib.h"
+#include "libcw.h"
 
 #include "i18n.h"
 #include "cmdline.h"
@@ -58,6 +58,14 @@ static int do_colors = TRUE;
 /* Curses windows used globally. */
 static WINDOW *text_box, *text_display, *timer_display;
 
+static const char *all_options = "s:|system,d:|device,"
+	"w:|wpm,t:|tone,v:|volume,"
+	"g:|gap,k:|weighting,"
+	"f:|infile,F:|outfile,"
+	"T:|time,"
+	/* "c:|colours,c:|colors,m|mono," */
+	"h|help,V|version";
+static cw_config_t *config;
 
 /*---------------------------------------------------------------------*/
 /*  Circular character queue                                           */
@@ -79,6 +87,23 @@ static volatile int queue_tail = 0,
 static volatile int is_queue_idle = TRUE;
 
 
+/* variables storing values of some command line arguments */
+static const char *argv0 = NULL;
+
+
+static int timer_get_practice_time(void);
+static int timer_set_practice_time(int practice_time);
+
+/*
+static int is_console = FALSE;
+static int is_soundcard = TRUE;
+static int is_alsa = FALSE;
+static char *console_device = NULL;
+static char *soundcard_device = NULL;
+static char *mixer_device = NULL;
+*/
+
+
 /*
  * queue_get_length()
  * queue_next_index()
@@ -352,12 +377,8 @@ timer_set_practice_time (int practice_time)
   return FALSE;
 }
 
-static void
-timer_get_practice_time_limits (int *min_time, int *max_time)
-{
-  *min_time = TIMER_MIN_TIME;
-  *max_time = TIMER_MAX_TIME;
-}
+
+
 
 
 /*
@@ -545,10 +566,7 @@ change_state_to_active (void)
 
   if (!is_sending_active)
     {
-      cw_flush_tone_queue ();
-      cw_queue_tone (20000, 500);
-      cw_queue_tone (20000, 1000);
-      cw_wait_for_tone_queue ();
+      cw_start_beep();
 
       /* Don't set sending_state until after the above warning has completed. */
       is_sending_active = TRUE;
@@ -595,12 +613,7 @@ change_state_to_idle (void)
       /* Remove everything in the outgoing character queue. */
       queue_discard_contents ();
 
-      cw_flush_tone_queue ();
-      cw_queue_tone (20000, 500);
-      cw_queue_tone (20000, 1000);
-      cw_queue_tone (20000, 500);
-      cw_queue_tone (20000, 1000);
-      cw_wait_for_tone_queue ();
+      cw_end_beep();
     }
 }
 
@@ -627,18 +640,18 @@ mode_buffer_random_text (moderef_t mode)
 
 
 /*
- * mode_cwlib_poll_sender()
+ * mode_libcw_poll_sender()
  *
  * Poll the CW library tone queue, and if it is getting low, arrange for
  * more data to be passed in to the sender.
  */
 static void
-mode_cwlib_poll_sender (void)
+mode_libcw_poll_sender (void)
 {
   if (cw_get_tone_queue_length () <= 1)
     {
       /*
-       * If sending is active, arrange more data for cwlib.  The source for
+       * If sending is active, arrange more data for libcw.  The source for
        * this data is dependent on the mode.  If in dictionary modes, update
        * and check the timer, then add more random data if the queue is empty.
        * If in keyboard mode, just dequeue anything currently on the character
@@ -688,20 +701,24 @@ mode_is_sending_active (void)
  * User interface introduction strings, split in two to avoid the 509
  * character limit imposed by ISO C89 on string literal lengths.
  */
-static const char *const INTRODUCTION = N_("\n"
-  "UNIX/Linux Morse Tutor v2.3, (C) 1997-2006 Simon Baldwin\n"
-  "--------------------------------------------------------\n\n"
+static const char *const INTRODUCTION = N_(
+  "UNIX/Linux Morse Tutor v3.0\n"
+  "Copyright (C) 1997-2006 Simon Baldwin\n"
+  "Copyright (C) 2011      Kamil Ignacak\n"
+  "---------------------------------------------------------\n"
   "Cwcp is an interactive Morse code tutor program, designed\n"
   "both for learning Morse code for the first time, and for\n"
   "experienced Morse users who want, or need, to improve\n"
-  "their receiving speed.\n\n");
-static const char *const INTRODUCTION_CONTINUED = N_("\n"
-  "To use the program, select a mode from those listed on\n"
-  "the left, and begin sending by pressing Return or F9.\n\n"
-  "You can vary the speed, tone, volume, and spacing of the\n"
-  "Morse code at any time using the appropriate keys.\n\n"
-  "To stop sending, press F9.  To stop the program, select\n"
-  "Exit from the Mode menu, or use F12 or ^C.\n");
+  "their receiving speed.\n");
+static const char *const INTRODUCTION_CONTINUED = N_(
+  "---------------------------------------------------------\n"
+  "Select mode:                   Up/Down arrow/F10/F11\n"
+  "Start sending selected mode:   Enter/F9\n"
+  "Pause:                         F9/Esc\n"
+  "Resume:                        F9\n"
+  "Exit program:                  menu->Exit/F12/^C\n"
+  "Use keys specified below to adjust speed, tone, volume,\n"
+  "and spacing of the Morse code at any time.\n");
 
 /* Alternative F-keys for folks without (some, or all) F-keys. */
 enum
@@ -725,10 +742,6 @@ enum
 /* User interface event loop running flag. */
 static int is_running = TRUE;
 
-/* Step values for the UI control of the CW parameters. */
-static const int STEP_WPM = 1, STEP_HZ = 100,
-                 STEP_VOL = 5, STEP_GAP = 1, STEP_TIME = 1;
-
 /* Color definitions. */
 static const short color_array[] = {
   COLOR_BLACK, COLOR_RED, COLOR_GREEN, COLOR_YELLOW,
@@ -1029,14 +1042,14 @@ interface_interpret (int c)
     case KEY_F (1):
     case PSEUDO_KEYF1:
     case KEY_LEFT:
-      if (cw_set_send_speed (cw_get_send_speed () - STEP_WPM))
+      if (cw_set_send_speed (cw_get_send_speed () - CW_SPEED_STEP))
         goto speed_update;
       break;
 
     case KEY_F (2):
     case PSEUDO_KEYF2:
     case KEY_RIGHT:
-      if (cw_set_send_speed (cw_get_send_speed () + STEP_WPM))
+      if (cw_set_send_speed (cw_get_send_speed () + CW_SPEED_STEP))
         goto speed_update;
       break;
 
@@ -1050,14 +1063,14 @@ interface_interpret (int c)
     case KEY_F (3):
     case PSEUDO_KEYF3:
     case KEY_END:
-      if (cw_set_frequency (cw_get_frequency () - STEP_HZ))
+      if (cw_set_frequency (cw_get_frequency () - CW_FREQUENCY_STEP))
         goto frequency_update;
       break;
 
     case KEY_F (4):
     case PSEUDO_KEYF4:
     case KEY_HOME:
-      if (cw_set_frequency (cw_get_frequency () + STEP_HZ))
+      if (cw_set_frequency (cw_get_frequency () + CW_FREQUENCY_STEP))
         goto frequency_update;
       break;
 
@@ -1070,13 +1083,13 @@ interface_interpret (int c)
 
     case KEY_F (5):
     case PSEUDO_KEYF5:
-      if (cw_set_volume (cw_get_volume () - STEP_VOL))
+      if (cw_set_volume (cw_get_volume () - CW_VOLUME_STEP))
         goto volume_update;
       break;
 
     case KEY_F (6):
     case PSEUDO_KEYF6:
-      if (cw_set_volume (cw_get_volume () + STEP_VOL))
+      if (cw_set_volume (cw_get_volume () + CW_VOLUME_STEP))
         goto volume_update;
       break;
 
@@ -1089,13 +1102,13 @@ interface_interpret (int c)
 
     case KEY_F (7):
     case PSEUDO_KEYF7:
-      if (cw_set_gap (cw_get_gap () - STEP_GAP))
+      if (cw_set_gap (cw_get_gap () - CW_GAP_STEP))
         goto gap_update;
       break;
 
     case KEY_F (8):
     case PSEUDO_KEYF8:
-      if (cw_set_gap (cw_get_gap () + STEP_GAP))
+      if (cw_set_gap (cw_get_gap () + CW_GAP_STEP))
         goto gap_update;
       break;
 
@@ -1109,13 +1122,13 @@ interface_interpret (int c)
 
     case KEY_NPAGE:
     case PSEUDO_KEYNPAGE:
-      if (timer_set_practice_time (timer_get_practice_time () - STEP_TIME))
+      if (timer_set_practice_time (timer_get_practice_time () - CW_PRACTICE_TIME_STEP))
         goto time_update;
       break;
 
     case KEY_PPAGE:
     case PSEUDO_KEYPPAGE:
-      if (timer_set_practice_time (timer_get_practice_time () + STEP_TIME))
+      if (timer_set_practice_time (timer_get_practice_time () + CW_PRACTICE_TIME_STEP))
         goto time_update;
       break;
 
@@ -1243,334 +1256,7 @@ interface_handle_event (int c)
 }
 
 
-/*---------------------------------------------------------------------*/
-/*  Command line mechanics                                             */
-/*---------------------------------------------------------------------*/
 
-/*
- * print_usage()
- *
- * Print out a brief message directing the user to the help function.
- */
-static void
-print_usage (const char *argv0)
-{
-  const char *format;
-
-  format = has_longopts ()
-    ? _("Try '%s --help' for more information.\n")
-    : _("Try '%s -h' for more information.\n");
-
-  fprintf (stderr, format, argv0);
-  exit (EXIT_FAILURE);
-}
-
-/*
- * print_help()
- *
- * Print out a brief page of help information.
- */
-static void
-print_help (const char *argv0)
-{
-  const char *format;
-  int min_speed, max_speed, min_frequency, max_frequency, min_volume,
-      max_volume, min_gap, max_gap, min_time, max_time;
-
-  cw_reset_send_receive_parameters ();
-  cw_get_speed_limits (&min_speed, &max_speed);
-  cw_get_frequency_limits (&min_frequency, &max_frequency);
-  cw_get_volume_limits (&min_volume, &max_volume);
-  cw_get_gap_limits (&min_gap, &max_gap);
-  timer_get_practice_time_limits (&min_time, &max_time);
-
-  format = has_longopts ()
-    ? _("Usage: %s [options...]\n\n"
-      "  -s, --sound=SOURCE     generate sound on SOURCE"
-      " [default 'soundcard']\n"
-      "                         one of 's[oundcard]', 'c[onsole]',"
-      " or 'b[oth]'\n"
-      "  -x, --sdevice=SDEVICE  use SDEVICE for soundcard [default %s]\n"
-      "  -y, --mdevice=MDEVICE  use MDEVICE for sound mixer [default %s]\n"
-      "  -d, --cdevice=CDEVICE  use CDEVICE for sound ioctl [default %s]\n")
-    : _("Usage: %s [options...]\n\n"
-      "  -s SOURCE   generate sound on SOURCE [default 'soundcard']\n"
-      "              one of 's[oundcard]', 'c[onsole]', or 'b[oth]'\n"
-      "  -x SDEVICE  use SDEVICE for soundcard [default %s]\n"
-      "  -y MDEVICE  use MDEVICE for sound mixer [default %s]\n"
-      "  -d CDEVICE  use CDEVICE for sound ioctl [default %s]\n");
-
-  printf (format, argv0,
-          cw_get_soundcard_file (),
-          cw_get_soundmixer_file (),
-          cw_get_console_file ());
-
-  format = has_longopts ()
-    ? _("  -i, --inifile=INIFILE  load practice words from INIFILE\n"
-      "  -w, --wpm=WPM          set initial words per minute [default %d]\n"
-      "                         valid WPM values are between %d and %d\n"
-      "  -t, --hz,--tone=HZ     set initial tone to HZ [default %d]\n"
-      "                         valid HZ values are between %d and %d\n"
-      "  -v, --volume=PERCENT   set initial volume to PERCENT [default %d]\n"
-      "                         valid PERCENT values are between %d and %d\n")
-    :
-      _("  -i INIFILE  load practice words from INIFILE\n"
-      "  -w WPM      set initial words per minute [default %d]\n"
-      "              valid WPM values are between %d and %d\n"
-      "  -t HZ       set initial tone to HZ [default %d]\n"
-      "              valid HZ values are between %d and %d\n"
-      "  -v PERCENT  set initial volume to PERCENT [default %d]\n"
-      "              valid PERCENT values are between %d and %d\n");
-
-  printf (format,
-          cw_get_send_speed (), min_speed, max_speed,
-          cw_get_frequency (), min_frequency, max_frequency,
-          cw_get_volume (), min_volume, max_volume);
-
-  format = has_longopts ()
-    ? _("  -g, --gap=GAP          set extra gap between letters [default %d]\n"
-      "                         valid GAP values are between %d and %d\n"
-      "  -p, --time=TIME        set initial practice time [default %d mins]\n"
-      "                         valid TIME values are between %d and %d\n"
-      "  -c, --colo[u]rs=CSET   set initial display colors where available\n"
-      "                         [default %d,%d,%d,%d]\n")
-    : _("  -g GAP      set extra gap between letters [default %d]\n"
-      "              valid GAP values are between %d and %d\n"
-      "  -p TIME     set initial practice time [default %d mins]\n"
-      "              valid TIME values are between %d and %d\n"
-      "  -c CSET     set initial display colors where available\n"
-      "              [default %d,%d,%d,%d]\n");
-
-  printf (format,
-          cw_get_gap (), min_gap, max_gap,
-          timer_get_practice_time (), min_time, max_time,
-          DISPLAY_FOREGROUND, DISPLAY_BACKGROUND,
-          BOX_FOREGROUND, BOX_BACKGROUND);
-
-  format = has_longopts ()
-    ? _("  -m, --mono             specify no colors [default colors]\n"
-      "  -h, --help             print this message\n"
-      "  -V, --version          output version information and exit\n\n")
-    : _("  -m          specify no colors [default colors]\n"
-      "  -h          print this message\n"
-      "  -V          output version information and exit\n\n");
-
-  printf (format);
-  exit (EXIT_SUCCESS);
-}
-
-
-/*
- * parse_command_line()
- *
- * Parse the command line options for initial values for the various
- * global and flag definitions.
- */
-static void
-parse_command_line (int argc, char **argv)
-{
-  const char *argv0;
-  int is_console = FALSE, is_soundcard = TRUE;
-  const char *console_device = NULL,
-             *soundcard_device = NULL, *mixer_device = NULL;
-  int option;
-  char *argument;
-
-  argv0 = program_basename (argv[0]);
-  while (get_option (argc, argv,
-                     _("s:|sound,d:|cdevice,x:|sdevice,y:|mdevice,i:|inifile,"
-                     "t:|tone,t:|hz,v:|volume,w:|wpm,g:|gap,p:|time,c:|colours,"
-                     "c:|colors,m|mono,h|help,V|version,#:|#"),
-                     &option, &argument))
-    {
-      int intarg;
-
-      switch (option)
-        {
-        case 's':
-          if (strcoll (argument, _("console")) == 0
-              || strcoll (argument, _("c")) == 0)
-            {
-              is_console = TRUE;
-              is_soundcard = FALSE;
-            }
-          else if (strcoll (argument, _("soundcard")) == 0
-                   || strcoll (argument, _("s")) == 0)
-            {
-              is_console = FALSE;
-              is_soundcard = TRUE;
-            }
-          else if (strcoll (argument, _("both")) == 0
-                   || strcoll (argument, _("b")) == 0)
-            {
-              is_console = TRUE;
-              is_soundcard = TRUE;
-            }
-          else
-            {
-              fprintf (stderr, _("%s: invalid sound source\n"), argv0);
-              exit (EXIT_FAILURE);
-            }
-          break;
-
-        case 'd':
-          console_device = argument;
-          break;
-
-        case 'x':
-          soundcard_device = argument;
-          break;
-
-        case 'y':
-          mixer_device = argument;
-          break;
-
-        case 'i':
-          if (!dictionary_load (argument))
-            {
-              fprintf (stderr, _("%s: error loading practice words\n"), argv0);
-              exit (EXIT_FAILURE);
-            }
-          break;
-
-        case '#':
-          if (!dictionary_write (argument))
-            {
-              fprintf (stderr, _("%s: error writing practice words\n"), argv0);
-              exit (EXIT_FAILURE);
-            }
-          break;
-
-        case 't':
-          if (sscanf (argument, "%d", &intarg) != 1
-              || !cw_set_frequency (intarg))
-            {
-              fprintf (stderr, _("%s: invalid tone value\n"), argv0);
-              exit (EXIT_FAILURE);
-            }
-          break;
-
-        case 'v':
-          if (sscanf (argument, "%d", &intarg) != 1
-              || !cw_set_volume (intarg))
-            {
-              fprintf (stderr, _("%s: invalid volume value\n"), argv0);
-              exit (EXIT_FAILURE);
-            }
-          break;
-
-        case 'w':
-          if (sscanf (argument, "%d", &intarg) != 1
-              || !cw_set_send_speed (intarg))
-            {
-              fprintf (stderr, _("%s: invalid wpm value\n"), argv0);
-              exit (EXIT_FAILURE);
-            }
-          break;
-
-        case 'g':
-          if (sscanf (argument, "%d", &intarg) != 1
-              || !cw_set_gap (intarg))
-            {
-              fprintf (stderr, _("%s: invalid gap value\n"), argv0);
-              exit (EXIT_FAILURE);
-            }
-          break;
-
-        case 'p':
-          if (sscanf (argument, "%d", &intarg) != 1
-              || !timer_set_practice_time (intarg))
-            {
-              fprintf (stderr, _("%s: invalid time value\n"), argv0);
-              exit (EXIT_FAILURE);
-            }
-          break;
-
-        case 'c':
-          if (sscanf (argument, "%d,%d,%d,%d",
-                      &display_foreground, &display_background,
-                      &box_foreground, &box_background) != 4
-              || display_foreground < 0 || display_foreground >= COLORS_COUNT
-              || display_background < 0 || display_background >= COLORS_COUNT
-              || box_foreground < 0 || box_foreground >= COLORS_COUNT
-              || box_background < 0 || box_background >= COLORS_COUNT)
-            {
-              fprintf (stderr, _("%s: invalid colors value\n"), argv0);
-              exit (EXIT_FAILURE);
-            }
-          break;
-
-        case 'm':
-          do_colors = FALSE;
-          break;
-
-        case 'h':
-          print_help (argv0);
-
-        case 'V':
-          printf (_("%s version %s, %s\n"),
-                  argv0, PACKAGE_VERSION, _(CW_COPYRIGHT));
-          exit (EXIT_SUCCESS);
-
-        case '?':
-          print_usage (argv0);
-
-        default:
-          fprintf (stderr, _("%s: getopts returned %c\n"), argv0, option);
-          exit (EXIT_FAILURE);
-        }
-
-    }
-  if (get_optind () != argc)
-    print_usage (argv0);
-
-  /* Deal with odd argument combinations. */
-  if (!is_console && console_device)
-    {
-      fprintf (stderr, _("%s: no console sound: -d invalid\n"), argv0);
-      print_usage (argv0);
-    }
-  if (!is_soundcard && soundcard_device)
-    {
-      fprintf (stderr, _("%s: no soundcard sound: -x invalid\n"), argv0);
-      print_usage (argv0);
-    }
-  if (!is_soundcard && mixer_device)
-    {
-      fprintf (stderr, _("%s: no soundcard sound: -y invalid\n"), argv0);
-      print_usage (argv0);
-    }
-
-  /* First set up soundcard sound if required. */
-  if (is_soundcard)
-    {
-      if (soundcard_device)
-        cw_set_soundcard_file (soundcard_device);
-      if (!cw_is_soundcard_possible ())
-        {
-          fprintf (stderr, _("%s: cannot set up soundcard sound\n"), argv0);
-          perror (cw_get_soundcard_file ());
-          exit (EXIT_FAILURE);
-        }
-      if (mixer_device)
-        cw_set_soundmixer_file (mixer_device);
-    }
-  cw_set_soundcard_sound (is_soundcard);
-
-  /* Now set up console sound, again if required. */
-  if (is_console)
-    {
-      if (console_device)
-        cw_set_console_file (console_device);
-      if (!cw_is_console_possible ())
-        {
-          fprintf (stderr, _("%s: cannot set up console sound\n"), argv0);
-          perror (cw_get_console_file ());
-          exit (EXIT_FAILURE);
-        }
-    }
-  cw_set_console_sound (is_console);
-}
 
 
 /*
@@ -1608,7 +1294,7 @@ poll_until_keypress_ready (int fd, int usecs)
         }
 
       /* Poll the sender on timeouts and on reads; it's just easier. */
-      mode_cwlib_poll_sender ();
+      mode_libcw_poll_sender ();
     }
   while (fd_count != 1);
 }
@@ -1637,53 +1323,96 @@ signal_handler (int signal_number)
  * Parse the command line, initialize a few things, then enter the main
  * program event loop, from which there is no return.
  */
-int
-main (int argc, char **argv)
+int main(int argc, char **argv)
 {
-  static const int SIGNALS[] = { SIGHUP, SIGINT, SIGQUIT, SIGPIPE, SIGTERM, 0 };
+	argv0 = program_basename(argv[0]);
 
-  int combined_argc, index;
-  char **combined_argv;
+	/* Set locale and message catalogs. */
+	i18n_initialize();
 
-  /* Set locale and message catalogs. */
-  i18n_initialize ();
+	/* Parse combined environment and command line arguments. */
+	int combined_argc;
+	char **combined_argv;
 
-  /* Parse combined environment and command line arguments. */
-  combine_arguments (_("CWCP_OPTIONS"),
-                     argc, argv, &combined_argc, &combined_argv);
-  parse_command_line (combined_argc, combined_argv);
+	/* Parse combined environment and command line arguments. */
+	combine_arguments(_("CWCP_OPTIONS"), argc, argv, &combined_argc, &combined_argv);
 
-  /* Set up signal handlers to clear up and exit on a range of signals. */
-  for (index = 0; SIGNALS[index] != 0; index++)
-    {
-      if (!cw_register_signal_handler (SIGNALS[index], signal_handler))
-        {
-          perror ("cw_register_signal_handler");
-          abort ();
-        }
-    }
+	config = cw_config_new();
+	if (!config) {
+		return -1;
+	}
+	config->has_practice_time = 1;
+	config->has_outfile = 1;
 
-  /*
-   * Build our table of modes from dictionaries, augmented with keyboard
-   * and any other local modes.
-   */
-  mode_initialize ();
+	if (!cw_process_argv(argc, argv, all_options, config)) {
+		fprintf(stderr, _("%s: failed to parse command line args\n"), argv0);
+		return EXIT_FAILURE;
+	}
+	if (!cw_config_is_valid(config)) {
+		fprintf(stderr, _("%s: inconsistent arguments\n"), argv0);
+		return EXIT_FAILURE;
+	}
 
-  /*
-   * Initialize the curses user interface, then catch and action every
-   * keypress we see.  Before calling getch, wait until data is available on
-   * stdin, polling the cwlib sender.  At 60WPM, a dot is 20ms, so polling
-   * for the maximum library speed needs a 10ms (10,000usec) timeout.
-   */
-  interface_initialize ();
-  while (is_running)
-    {
-      poll_until_keypress_ready (fileno (stdin), 10000);
-      interface_handle_event (getch ());
-    }
+	if (config->input_file) {
+		if (!dictionary_load(config->input_file)) {
+			fprintf(stderr, _("%s: %s\n"), argv0, strerror(errno));
+			fprintf(stderr, _("%s: can't load dictionary from input file %s\n"), argv0, config->input_file);
+			return EXIT_FAILURE;
+		}
+	}
 
-  /* Clean up and return. */
-  interface_destroy ();
-  cw_wait_for_tone_queue ();
-  return EXIT_SUCCESS;
+	if (config->output_file) {
+		if (!dictionary_write(config->output_file)) {
+			fprintf(stderr, _("%s: %s\n"), argv0, strerror(errno));
+			fprintf(stderr, _("%s: can't save dictionary to output file  %s\n"), argv0, config->input_file);
+			return EXIT_FAILURE;
+		}
+	}
+
+	if (!cw_generator_new_from_config(config, argv0)) {
+		fprintf(stderr, "%s: failed to create generator\n", argv0);
+		return EXIT_FAILURE;
+	}
+	timer_set_practice_time(config->practice_time);
+
+
+	int index;
+	static const int SIGNALS[] = { SIGHUP, SIGINT, SIGQUIT, SIGPIPE, SIGTERM, 0 };
+	/* Set up signal handlers to clear up and exit on a range of signals. */
+	for (index = 0; SIGNALS[index] != 0; index++) {
+		if (!cw_register_signal_handler(SIGNALS[index], signal_handler)) {
+			fprintf(stderr, _("%s: can't register signal: %s\n"), argv0, strerror(errno));
+			return EXIT_FAILURE;
+		}
+	}
+
+	/*
+	 * Build our table of modes from dictionaries, augmented with keyboard
+	 * and any other local modes.
+	 */
+	mode_initialize ();
+
+	/*
+	 * Initialize the curses user interface, then catch and action every
+	 * keypress we see.  Before calling getch, wait until data is available on
+	 * stdin, polling the libcw sender.  At 60WPM, a dot is 20ms, so polling
+	 * for the maximum library speed needs a 10ms (10,000usec) timeout.
+	 */
+	interface_initialize ();
+	cw_generator_start();
+	while (is_running) {
+		poll_until_keypress_ready(fileno(stdin), 10000);
+		interface_handle_event(getch());
+	}
+
+	/* Clean up and return. */
+	interface_destroy();
+	cw_wait_for_tone_queue();
+	cw_generator_stop();
+	/* Reset to ensure that the mixer volume gets restored. */
+	cw_complete_reset();
+	cw_generator_delete();
+	cw_config_delete(&config);
+
+	return EXIT_SUCCESS;
 }
