@@ -1,6 +1,6 @@
-/* vi: set ts=2 shiftwidth=2 expandtab:
- *
+/*
  * Copyright (C) 2001-2006  Simon Baldwin (simon_baldwin@yahoo.com)
+ * Copyright (C) 2011       Kamil Ignacak (acerion@wp.pl)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,28 +12,33 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifndef _CWCMDLINE_H
 #define _CWCMDLINE_H
 
+#include <stdbool.h>
+#include "cw_common.h" /* cw_config_t */
+
 #if defined(__cplusplus)
 extern "C" {
 #endif
 
-extern const char *program_basename (const char *argv0);
-extern void combine_arguments (const char *env_variable,
-                               int argc, char *const argv[],
-                               int *new_argc, char **new_argv[]);
 
-extern int has_longopts (void);
-extern int get_option (int argc, char *const argv[],
+extern const char *program_basename(const char *argv0);
+extern void combine_arguments(const char *env_variable,
+			      int argc, char *const argv[],
+			      int *new_argc, char **new_argv[]);
+
+extern int cw_process_argv(int argc, char *const argv[], const char *options, cw_config_t *config);
+extern bool has_longopts(void);
+extern int get_option(int argc, char *const argv[],
                        const char *descriptor,
                        int *option, char **argument);
-extern int get_optind (void);
+extern int get_optind(void);
 
 #if defined(__cplusplus)
 }
