@@ -1,6 +1,6 @@
-/* vi: set ts=2 shiftwidth=2 expandtab:
- *
+/*
  * Copyright (C) 2001-2006  Simon Baldwin (simon_baldwin@yahoo.com)
+ * Copyright (C) 2011       Kamil Ignacak (acerion@wp.pl)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -12,20 +12,23 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifndef _CWCOPYRIGHT_H
 #define _CWCOPYRIGHT_H
 
+#include "i18n.h"
+
 #if defined(__cplusplus)
 extern "C" {
 #endif
 
 static const char *const CW_COPYRIGHT =
-  N_("Copyright (C) 2001-2006  Simon Baldwin\n\n"
+  N_("Copyright (C) 2001-2006  Simon Baldwin\n"
+  "Copyright (C) 2011       Kamil Ignacak\n\n"
   "This program comes with ABSOLUTELY NO WARRANTY; for details please see\n"
   "the file 'COPYING' supplied with the source code.  This is free software,\n"
   "and you are welcome to redistribute it under certain conditions; again,\n"
