@@ -1,6 +1,5 @@
-// vi: set ts=2 shiftwidth=2 expandtab:
-//
 // Copyright (C) 2001-2006  Simon Baldwin (simon_baldwin@yahoo.com)
+// Copyright (C) 2011       Kamil Ignacak (acerion@wp.pl)
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -12,15 +11,18 @@
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+// You should have received a copy of the GNU General Public License along
+// with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 //
 
 #ifndef _XCWCP_APPLICATION_H
 #define _XCWCP_APPLICATION_H
 
-#include <qmainwindow.h>
+#include <QMainWindow>
+#include <QToolButton>
+#include <QComboBox>
+#include <QSpinBox>
 
 #include <string>
 #include <deque>
@@ -28,13 +30,6 @@
 #include "modeset.h"
 
 
-class QKeyEvent;
-class QComboBox;
-class QSpinBox;
-class QToolButton;
-class QCheckBox;
-class QTimer;
-
 
 //-----------------------------------------------------------------------
 //  Class Application
@@ -53,19 +48,18 @@ class Application : public QMainWindow
 {
  Q_OBJECT
  public:
-  Application ();
+  Application();
 
   // Handle key press and mouse button press events
   void key_event (QKeyEvent *event);
   void mouse_event (QMouseEvent *event);
-
  protected:
   void closeEvent (QCloseEvent *event);
 
  private slots:
   // Qt widget library callback functions.
   void about ();
-  void startstop ();
+  void startstop();
   void start ();
   void stop ();
   void new_instance ();
@@ -80,25 +74,51 @@ class Application : public QMainWindow
   void adaptive_receive_change ();
   void fonts ();
   void colors ();
+  void toggle_toolbar(void);
   void poll_timer_event ();
 
  private:
-  // Class variable to enable sharing of the cwlib across instances.  Set to
+  // Class variable to enable sharing of the libcw across instances.  Set to
   // the 'this' of the CW user instance, or NULL if no current user.
-  static Application *cwlib_user_application_instance;
+  static Application *libcw_user_application_instance;
 
+  QPixmap xcwcp_icon;
+
+  bool play_;
+  QPixmap start_icon;
+  QPixmap stop_icon;
   // GUI elements used throughout the class.
+  QToolBar *toolbar; // main toolbar
+
+  QToolButton *startstop_button_;
+  QAction *startstop_; // Shared between toolbar and Progam menu
   QComboBox *mode_combo_;
   QSpinBox *speed_spin_;
   QSpinBox *frequency_spin_;
   QSpinBox *volume_spin_;
   QSpinBox *gap_spin_;
-  QToolButton *startstop_button_;
-  QCheckBox *reverse_paddles_;
-  QCheckBox *curtis_mode_b_;
-  QCheckBox *adaptive_receive_;
+
+
+  QMenu *program_menu_;
+  QAction *new_window_;
+  QAction *clear_display_;
+  QAction *sync_speed_;
+  QAction *close_;
+  QAction *quit_;
+
+  QMenu *settings_;
+  QAction *reverse_paddles_;
+  QAction *curtis_mode_b_;
+  QAction *adaptive_receive_;
+  QAction *font_settings_;
+  QAction *color_settings_;
+  QAction *toolbar_visibility_;
+
+  QMenu *help_;
+  QAction *about_;
+
   Display *display_;
-  QPopupMenu *file_menu_;
+
   int file_synchronize_speed_id_;
   int file_start_id_;
   int file_stop_id_;
@@ -113,21 +133,33 @@ class Application : public QMainWindow
 
   // Poll timer, used to ensure that all of the application processing can
   // be handled in the foreground, rather than in the signal handling context
-  // of a cwlib tone queue low callback.
+  // of a libcw tone queue low callback.
   QTimer *poll_timer_;
 
-  // Flag indicating if this instance is currently using the cwlib.
-  bool is_using_cwlib_;
+  // Flag indicating if this instance is currently using the libcw.
+  bool is_using_libcw_;
 
   // Saved receive speed, used to reinstate adaptive tracked speed on start.
   int saved_receive_speed_;
 
-  // Keying callback function for cwlib.  There is a static version for
+  // Keying callback function for libcw.  There is a static version for
   // the whole class, and an instance version for each object.  The class
   // version calls the relevant instance version, based on which instance is
-  // the current registered cwlib user.
-  static void cwlib_keying_event_static (void *, int key_state);
-  void cwlib_keying_event (int key_state);
+  // the current registered libcw user.
+  static void libcw_keying_event_static (void *, int key_state);
+  void libcw_keying_event (int key_state);
+
+  // Wrappers for creating UI.
+  void make_central_widget(void);
+  void make_toolbar(void);
+  void make_mode_combo(void);
+  void make_program_menu(void);
+  void make_settings_menu(void);
+  void make_help_menu(void);
+
+  void make_auxiliaries_begin(void);
+  void make_auxiliaries_end(void);
+
 
   // Prevent unwanted operations.
   Application (const Application &);
