@@ -1,6 +1,5 @@
-// vi: set ts=2 shiftwidth=2 expandtab:
-//
 // Copyright (C) 2001-2006  Simon Baldwin (simon_baldwin@yahoo.com)
+// Copyright (C) 2011       Kamil Ignacak (acerion@wp.pl)
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -12,25 +11,25 @@
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+// You should have received a copy of the GNU General Public License along
+// with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 //
 
 #include "../config.h"
 
+#include <cstdio>
 #include <cstdlib>
 #include <string>
 #include <deque>
 #include <sstream>
 
-#include <qevent.h>
 
 #include "sender.h"
 #include "display.h"
 #include "modeset.h"
 
-#include "cwlib.h"
+#include "libcw.h"
 
 #include "i18n.h"
 
@@ -81,7 +80,7 @@ Sender::handle_key_event (QKeyEvent *event, const Mode *current_mode)
         {
           // If the key was backspace, remove the last queued character, or at
           // least try, and we are done.
-          if (event->key () == QEvent::Key_Backspace)
+          if (event->key () == Qt::Key_Backspace)
             {
               delete_character ();
               event->accept ();
@@ -90,13 +89,13 @@ Sender::handle_key_event (QKeyEvent *event, const Mode *current_mode)
             {
               // Extract the ASCII keycode from the key event, and queue the
               // character for sending, converted to uppercase.
-              const char c = event->ascii ();
-              enqueue_string (std::string (1, c));
+		    const char *c = event->text().toAscii().data();
+		    enqueue_string(c);
 
               // Accept the event if the character was sendable.  If not, it
               // won't have queued, and so by ignoring it we can let
               // characters such as Tab pass up to the parent.
-              if (cw_check_character (toupper (c)))
+              if (cw_check_character (toupper (c[0])))
                 event->accept ();
             }
         }
@@ -121,38 +120,46 @@ Sender::clear ()
 // Called when the CW send buffer is empty.  If the queue is not idle, take
 // the next character from the queue and send it.  If there are no more queued
 // characters, set the queue to idle.
-void
-Sender::dequeue_character ()
+void Sender::dequeue_character()
 {
-  if (!is_queue_idle_)
-    {
-      if (!send_queue_.empty ())
-        {
-          // Take the next character off the queue and send it.  We don't
-          // expect sending to fail as only sendable characters are queued.
-          const char c = toupper (send_queue_.front ());
-          send_queue_.pop_front ();
-          if (!cw_send_character (c))
-            {
-              perror ("cw_send_character");
-              abort ();
-            }
+	if (is_queue_idle_) {
+		return;
+	}
 
-          // Update the status bar with the character being sent.
-          std::ostringstream outs;
-          outs << _("Sending '")
-               << c << _("' at ") << cw_get_send_speed () << _(" WPM");
-          display_->show_status (outs.str ());
-        }
-      else
-        {
-          is_queue_idle_ = true;
-          display_->clear_status ();
-        }
-    }
+	if (send_queue_.empty()) {
+		is_queue_idle_ = true;
+		display_->clear_status();
+		return;
+	}
+
+	// Take the next character off the queue and send it.  We don't
+	// expect sending to fail as only sendable characters are queued.
+	const char c = toupper(send_queue_.front());
+	send_queue_.pop_front();
+	if (!cw_send_character(c)) {
+		perror("cw_send_character");
+		abort();
+	}
+
+	// Update the status bar with the character being sent.
+	// I'm sure there is a way to create QString in one line,
+	// without series of concatenations. For now I'll use C string.
+	// Put the sent char at the end to avoid "jumping" of whole
+	// string when width of glyph of sent char changes at variable
+	// font width.
+	const char *format = _("Sending at %d WPM: '%c'");
+	char c_status[100]; // TODO: dynamic array size in C99 (strlen(format) + 1 + X)
+	snprintf(c_status, strlen(format) + 1, format, cw_get_send_speed(), c);
+	QString status(c_status);
+	display_->show_status(status);
+
+	return;
 }
 
 
+
+
+
 // enqueue_string()
 //
 // Queues a string for sending by the CW sender.  Rejects any unsendable
