@@ -1,6 +1,5 @@
-// vi: set ts=2 shiftwidth=2 expandtab:
-//
 // Copyright (C) 2001-2006  Simon Baldwin (simon_baldwin@yahoo.com)
+// Copyright (C) 2011       Kamil Ignacak (acerion@wp.pl)
 //
 // This program is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License
@@ -12,18 +11,20 @@
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+// You should have received a copy of the GNU General Public License along
+// with this program; if not, write to the Free Software Foundation, Inc.,
+// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 //
 
 #ifndef _XCWCP_SENDER_H
 #define _XCWCP_SENDER_H
 
+#include <QKeyEvent>
+
 #include <string>
 #include <deque>
 
-class QKeyEvent;
+
 
 
 //-----------------------------------------------------------------------
@@ -56,7 +57,7 @@ class Sender
   Display *display_;
 
   // Deque and queue manipulation functions, used to handle and maintain the
-  // buffer of characters awaiting cwlib send.
+  // buffer of characters awaiting libcw send.
   bool is_queue_idle_;
   std::deque<char> send_queue_;
   void dequeue_character ();
